@import Cocoa;

#import "TextStorageSwiftBridging-Swift.h"

static CFTimeInterval MeasureDuration(void (^block)(void)) {
   CFTimeInterval start = CACurrentMediaTime();
   block();
   CFTimeInterval end = CACurrentMediaTime();
   return end - start;
}

void RunSwiftBridgingPerformanceTest(void) {
   NSTextStorage *objcTextStorage = [[NSTextStorage alloc] init];
   NSTextStorage *swiftTextStorage = [[TextStorageSwiftSubclass alloc] init];

   NSMutableString *string = [[NSMutableString alloc] init];
   for (NSUInteger i = 0; i < 100000; i++) {
      [string appendString:@"0123456789"];
   }

   NSDictionary<NSAttributedStringKey,id> *attributes = @{NSFontAttributeName: [NSFont systemFontOfSize:12]};
   NSAttributedString *attributedString = [[NSAttributedString alloc] initWithString:string attributes:attributes];

   [objcTextStorage appendAttributedString:attributedString];
   [swiftTextStorage appendAttributedString:attributedString];

   CFTimeInterval objcDuration = MeasureDuration(^{
      for (NSUInteger idx = 0; idx < objcTextStorage.length; idx++) {
         [objcTextStorage attributesAtIndex:idx effectiveRange:nil];
      }
   });
   NSLog(@"Objective-C: %g", objcDuration);

   CFTimeInterval swiftDuration = MeasureDuration(^{
      for (NSUInteger idx = 0; idx < objcTextStorage.length; idx++) {
         [swiftTextStorage attributesAtIndex:idx effectiveRange:nil];
      }
   });
   NSLog(@"Swift: %g", swiftDuration);
}
